package opengles2.tutorial5;

import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.Matrix;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

public class MainRenderer implements GLSurfaceView.Renderer {
    int width, height;

    private final float[] projMatrix = new float[16];
    private final float[] viewMatrix = new float[16];
    private final float[] basicMatrix = new float[16];
    private float[] modelMatrix = new float[16];

    ObjSquare objsquare;

    public MainRenderer() {
        this.objsquare = new ObjSquare();
    }

    @Override
    public void onSurfaceCreated(GL10 unused, EGLConfig config) {
        GLES20.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

        this.objsquare.SetProgramShader();
    }

    @Override
    public void onSurfaceChanged(GL10 unused, int width, int height) {
        this.width = width;
        this.height = height;

        GLES20.glViewport(0, 0, this.width, this.height);
        Matrix.orthoM(this.projMatrix, 0, 0.0f, this.width - 1.0f, 0.0f, this.height - 1.0f, 1.0f, -1.0f);
        Matrix.setLookAtM(this.viewMatrix, 0, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);
        Matrix.multiplyMM(this.basicMatrix, 0, this.projMatrix, 0, this.viewMatrix, 0);
    }

    @Override
    public void onDrawFrame(GL10 unused) {
        GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);

        this.ResetModelMatrix();
        Matrix.translateM(this.modelMatrix, 0, this.width / 4.0f, this.height / 4.0f * 3.0f, 0.0f);
        Matrix.scaleM(this.modelMatrix, 0, 50.0f, 50.0f, 0.0f);
        this.objsquare.DrawSquare(this.modelMatrix, 1);

        this.ResetModelMatrix();
        Matrix.translateM(this.modelMatrix, 0, this.width / 4.0f * 3.0f, this.height / 4.0f * 3.0f, 0.0f);
        Matrix.scaleM(this.modelMatrix, 0, 100.0f, 100.0f, 0.0f);
        this.objsquare.DrawSquare(this.modelMatrix, 2);

        this.ResetModelMatrix();
        Matrix.translateM(this.modelMatrix, 0, this.width / 4.0f, this.height / 4.0f, 0.0f);
        Matrix.scaleM(this.modelMatrix, 0, 150.0f, 150.0f, 0.0f);
        this.objsquare.DrawSquare(this.modelMatrix, 3);

        this.ResetModelMatrix();
        Matrix.translateM(this.modelMatrix, 0, this.width / 4.0f * 3.0f, this.height / 4.0f, 0.0f);
        Matrix.scaleM(this.modelMatrix, 0, 200.0f, 200.0f, 0.0f);
        this.objsquare.DrawSquare(this.modelMatrix, 0);
    }

    public static int LoadShader(int type, String shaderCode) {
        int shader = GLES20.glCreateShader(type);
        GLES20.glShaderSource(shader, shaderCode);
        GLES20.glCompileShader(shader);

        return shader;
    }

    private void ResetModelMatrix() {
        this.modelMatrix = this.basicMatrix.clone();
    }

}
