package opengles2.tutorial5;

import android.opengl.GLES20;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import javax.microedition.khronos.opengles.GL10;

public class ObjSquare {
    private FloatBuffer vertexBuffer;
    private ShortBuffer indexBuffer;

    private final int COORDS_PER_VERTEX = 2;
    private final int vertexStride = COORDS_PER_VERTEX * 4;
    private float[] vertices = {
            -0.5f, -0.5f,
            0.5f, -0.5f,
            -0.5f, 0.5f,
            0.5f, 0.5f
    };

    private short[] indices = {
            0, 1, 2,
            2, 1, 3
    };

    private float[] color_red = { 1.0f, 0.0f, 0.0f, 1.0f };
    private float[] color_green = { 0.0f, 1.0f, 0.0f, 1.0f };
    private float[] color_blue = { 0.0f, 0.0f, 1.0f, 1.0f };
    private float[] color_white = { 1.0f, 1.0f, 1.0f, 1.0f };

    private final String vertexShaderCode =
            "uniform mat4 uMVPMatrix;" +
            "attribute vec4 vPosition;" +
            "void main() {" +
            "  gl_Position = uMVPMatrix * vPosition;" +
            "}";

    private final String fragmentShaderCode =
            "precision mediump float;" +
            "uniform vec4 vColor;" +
            "void main() {" +
            "  gl_FragColor = vColor;" +
            "}";

    private int mProgram;

    public ObjSquare() {
        ByteBuffer vbb = ByteBuffer.allocateDirect(this.vertices.length * 4);
        vbb.order(ByteOrder.nativeOrder());
        this.vertexBuffer = vbb.asFloatBuffer();
        this.vertexBuffer.put(this.vertices);
        this.vertexBuffer.position(0);

        ByteBuffer ibb = ByteBuffer.allocateDirect(this.indices.length * 2);
        ibb.order(ByteOrder.nativeOrder());
        this.indexBuffer = ibb.asShortBuffer();
        this.indexBuffer.put(this.indices);
        this.indexBuffer.position(0);
    }
    public void SetProgramShader() {
        int vertexShader = MainRenderer.LoadShader(GLES20.GL_VERTEX_SHADER, this.vertexShaderCode);
        int fragmentShader = MainRenderer.LoadShader(GLES20.GL_FRAGMENT_SHADER, this.fragmentShaderCode);

        this.mProgram = GLES20.glCreateProgram();
        GLES20.glAttachShader(this.mProgram, vertexShader);
        GLES20.glAttachShader(this.mProgram, fragmentShader);
        GLES20.glLinkProgram(this.mProgram);
    }

    private int positionHandle;
    private int colorHandle;
    private int modelHandle;

    public void DrawSquare(float[] modelMatrix, int color) {
        GLES20.glUseProgram(this.mProgram);

        this.positionHandle = GLES20.glGetAttribLocation(this.mProgram, "vPosition");
        GLES20.glEnableVertexAttribArray(this.positionHandle);
        GLES20.glVertexAttribPointer(this.positionHandle, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, this.vertexStride, this.vertexBuffer);

        this.colorHandle = GLES20.glGetUniformLocation(this.mProgram, "vColor");
        if (color == 1) // Red.
            GLES20.glUniform4fv(this.colorHandle, 1, this.color_red, 0);
        else if (color == 2) // Green.
            GLES20.glUniform4fv(this.colorHandle, 1, this.color_green, 0);
        else if (color == 3) // Blue.
            GLES20.glUniform4fv(this.colorHandle, 1, this.color_blue, 0);
        else // White.
            GLES20.glUniform4fv(this.colorHandle, 1, this.color_white, 0);

        this.modelHandle = GLES20.glGetUniformLocation(this.mProgram, "uMVPMatrix");
        GLES20.glUniformMatrix4fv(this.modelHandle, 1, false, modelMatrix, 0);

        GLES20.glDrawElements(GL10.GL_TRIANGLES, this.indices.length, GL10.GL_UNSIGNED_SHORT, this.indexBuffer);

        GLES20.glDisableVertexAttribArray(this.positionHandle);
    }

}
